package com.qmth.themis.backend.api;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qmth.themis.business.annotation.ApiJsonObject;
import com.qmth.themis.business.annotation.ApiJsonProperty;
import com.qmth.themis.business.base.BasePage;
import com.qmth.themis.business.config.SystemConfig;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.dto.response.TBExamInvigilateUserDto;
import com.qmth.themis.business.entity.*;
import com.qmth.themis.business.enums.*;
import com.qmth.themis.business.service.TBAttachmentService;
import com.qmth.themis.business.service.TBExamInvigilateUserService;
import com.qmth.themis.business.service.TBTaskHistoryService;
import com.qmth.themis.business.util.JacksonUtil;
import com.qmth.themis.business.util.ServletUtil;
import com.qmth.themis.common.enums.ExceptionResultEnum;
import com.qmth.themis.common.exception.BusinessException;
import com.qmth.themis.common.util.Result;
import com.qmth.themis.common.util.ResultUtil;
import com.qmth.themis.business.dto.MqDto;
import com.qmth.themis.business.enums.MqTagEnum;
import com.qmth.themis.business.enums.MqTopicEnum;
import com.qmth.themis.business.service.MqDtoService;
import io.swagger.annotations.*;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description: 监考员设置 前端控制器
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/6/25
 */
@Api(tags = "监考员设置Controller")
@RestController
@RequestMapping("/${prefix.url.admin}/invigilateUser")
public class TBExamInvigilateUserController {

    @Resource
    TBExamInvigilateUserService tbExamInvigilateUserService;

    @Resource
    MqDtoService mqDtoService;

    @Resource
    TBTaskHistoryService taskHistoryService;

    @Resource
    TBAttachmentService tbAttachmentService;

    @Resource
    SystemConfig systemConfig;

    @ApiOperation(value = "监考员查询接口")
    @RequestMapping(value = "/query", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "监考员信息", response = TBExamInvigilateUserDto.class)})
    public Result query(@ApiParam(value = "考场代码", required = false) @RequestParam(required = false) String roomCode, @ApiParam(value = "用户id", required = false) @RequestParam(required = false) Long userId, @ApiParam(value = "分页页码", required = true) @RequestParam int pageNumber, @ApiParam(value = "分页数", required = true) @RequestParam int pageSize) {
        IPage<TBExamInvigilateUserDto> examInvigilateUserDtoIPage = tbExamInvigilateUserService.examInvigilateUserQuery(new Page<>(pageNumber, pageSize), roomCode, userId);
        BasePage basePage = new BasePage(examInvigilateUserDtoIPage.getRecords(), examInvigilateUserDtoIPage.getCurrent(), examInvigilateUserDtoIPage.getSize(), examInvigilateUserDtoIPage.getTotal());
        Map map = new HashMap<>();
        map.put(SystemConstant.RECORDS, basePage);
        return ResultUtil.ok(map);
    }

    @ApiOperation(value = "监考员修改接口")
    @RequestMapping(value = "/save", method = RequestMethod.POST)
    @Transactional
    @ApiResponses({@ApiResponse(code = 200, message = "{\"success\":true}", response = Result.class)})
    public Result save(
            @ApiJsonObject(name = "saveInvigilateUser", value = {
                    @ApiJsonProperty(key = "roomCode", description = "考场代码"),
                    @ApiJsonProperty(key = "userId", type = "long", example = "1", description = "用户id")
            })
            @ApiParam(value = "监考员信息", required = true) @RequestBody Map<String, Object> mapParameter) {
        String roomCode = null;
        try {
            if (Objects.isNull(mapParameter.get("roomCode")) || Objects.equals(mapParameter.get("roomCode"), "")) {
                throw new BusinessException(ExceptionResultEnum.ROOMCODE_IS_NULL);
            }
            roomCode = String.valueOf(mapParameter.get("roomCode"));
            if (Objects.isNull(mapParameter.get("userId")) || Objects.equals(mapParameter.get("userId"), "")) {
                throw new BusinessException(ExceptionResultEnum.USER_ID_IS_NULL);
            }
            String userId = String.valueOf(mapParameter.get("userId"));
            String[] userIds = userId.split(",");
            if (userIds.length > 3) {
                throw new BusinessException(ExceptionResultEnum.ERROR.getStatusCode(), ExceptionResultEnum.ERROR.getCode(), "一个考场最多设置3名监考员");
            }
            TBOrg tbOrg = (TBOrg) ServletUtil.getRequestOrg();
            QueryWrapper<TBExamInvigilateUser> tbExamInvigilateUserQueryWrapper = new QueryWrapper<>();
            tbExamInvigilateUserQueryWrapper.lambda().eq(TBExamInvigilateUser::getRoomCode, roomCode);
            if (Objects.nonNull(tbOrg) && Objects.isNull(tbOrg.getId())) {
                throw new BusinessException("监考员机构id不允许为空");
            }
            Long orgId = tbOrg.getId();
            List<TBExamInvigilateUser> tbExamInvigilateUserList = tbExamInvigilateUserService.list(tbExamInvigilateUserQueryWrapper);
            List<String> roomNameList = tbExamInvigilateUserList.stream().map(s -> s.getRoomName()).collect(Collectors.toList());
            tbExamInvigilateUserService.remove(tbExamInvigilateUserQueryWrapper);
            if (userIds.length > 0) {
                tbExamInvigilateUserList = new ArrayList<>();
                for (int i = 0; i < userIds.length; i++) {
                    TBExamInvigilateUser tbExamInvigilateUser = new TBExamInvigilateUser(orgId, Long.parseLong(userIds[i]), roomCode, roomNameList.get(0));
                    tbExamInvigilateUserList.add(tbExamInvigilateUser);
                }
                tbExamInvigilateUserService.saveBatch(tbExamInvigilateUserList);
            }
        } catch (Exception e) {
            e.printStackTrace();
            if (e instanceof DuplicateKeyException) {
                String errorColumn = e.getCause().toString();
                String columnStr = errorColumn.substring(errorColumn.lastIndexOf("key") + 3, errorColumn.length()).replaceAll("'", "");
                throw new BusinessException("考场代码[" + roomCode + "]下的" + FieldUniqueEnum.convertToCode(columnStr) + "数据不允许重复插入");
            } else if (e instanceof BusinessException) {
                throw new BusinessException(e.getMessage());
            } else {
                throw new RuntimeException(e);
            }
        }
        return ResultUtil.ok(JacksonUtil.parseJson(SystemConstant.SUCCESS));
    }

    @ApiOperation(value = "监考员导入接口")
    @RequestMapping(value = "/import", method = RequestMethod.POST)
    @Transactional
    @ApiResponses({@ApiResponse(code = 200, message = "{\"taskId\":0}", response = Result.class)})
    public Result importData(@ApiParam(value = "上传文件", required = true) @RequestParam MultipartFile file) {
        if (Objects.isNull(file) || Objects.equals(file, "")) {
            throw new BusinessException(ExceptionResultEnum.ATTACHMENT_IS_NULL);
        }
        Map<String, Object> mapParameter = systemConfig.getOssEnv(UploadFileEnum.file.getId());
        TBAttachment tbAttachment = null;
        TBTaskHistory tbTaskHistory = null;
        Map transMap = new HashMap();
        try {
            TBUser tbUser = (TBUser) ServletUtil.getRequestAccount();
            tbAttachment = tbAttachmentService.saveAttachment(file, ServletUtil.getRequestMd5(), ServletUtil.getRequestPath(), mapParameter, tbUser.getOrgId(), tbUser.getId());
            if (Objects.isNull(tbAttachment)) {
                throw new BusinessException(ExceptionResultEnum.ATTACHMENT_ERROR);
            } else {
                //往任务表里插一条数据
                tbTaskHistory = new TBTaskHistory(TaskTypeEnum.IMPORT_INVIGILATE_USER, tbAttachment.getId(), TaskStatusEnum.INIT, SystemConstant.IMPORT_INIT, 0d, tbAttachment.getName(), tbAttachment.getRemark(), tbUser.getId());
                taskHistoryService.save(tbTaskHistory);
                transMap.put("tbTaskHistory", tbTaskHistory);
            }
            transMap.put("createId", tbUser.getId());
            transMap.put("orgId", tbUser.getOrgId());
            transMap.put("remark", tbAttachment.getRemark());
            //mq发送消息start
            MqDto mqDto = new MqDto(MqTopicEnum.themisTopic.getCode(), MqTagEnum.roomCodeImport.name(), transMap, MqTagEnum.roomCodeImport, String.valueOf(tbTaskHistory.getId()), tbUser.getName());
            mqDtoService.assembleSendOneWayMsg(mqDto);
            //mq发送消息end
        } catch (Exception e) {
            e.printStackTrace();
            if (Objects.nonNull(tbAttachment)) {
                tbAttachmentService.deleteAttachment(mapParameter, tbAttachment);
            }
            if (e instanceof BusinessException) {
                throw new BusinessException(e.getMessage());
            } else {
                throw new RuntimeException(e);
            }
        }
        Map map = new HashMap();
        map.put(SystemConstant.TASK_ID, tbTaskHistory.getId());
        return ResultUtil.ok(map);
    }

    @ApiOperation(value = "监考员导出接口")
    @RequestMapping(value = "/export", method = RequestMethod.POST)
    @Transactional
    @ApiResponses({@ApiResponse(code = 200, message = "{\"taskId\":0}", response = Result.class)})
    public Result export() {
        TBTaskHistory tbTaskHistory = null;
        try {
            int count = tbExamInvigilateUserService.count();
            if (count > 0) {
                Map transMap = new HashMap();
                TBUser tbUser = (TBUser) ServletUtil.getRequestAccount();
                //往任务表里插一条数据
                tbTaskHistory = new TBTaskHistory(TaskTypeEnum.EXPORT_INVIGILATE_USER, TaskStatusEnum.INIT, SystemConstant.EXPORT_INIT, 0d, tbUser.getId());
                taskHistoryService.save(tbTaskHistory);
                transMap.put("tbTaskHistory", tbTaskHistory);
                transMap.put("createId", tbUser.getId());
                transMap.put("orgId", tbUser.getOrgId());
                //mq发送消息start
                MqDto mqDto = new MqDto(MqTopicEnum.themisTopic.getCode(), MqTagEnum.roomCodeExport.name(), transMap, MqTagEnum.roomCodeExport, String.valueOf(tbTaskHistory.getId()), tbUser.getName());
                mqDtoService.assembleSendOneWayMsg(mqDto);
                //mq发送消息end
            } else {
                return ResultUtil.ok(JacksonUtil.parseJson(SystemConstant.EXPORT_NO_DATA));
            }
        } catch (Exception e) {
            e.printStackTrace();
            if (e instanceof BusinessException) {
                throw new BusinessException(e.getMessage());
            } else {
                throw new RuntimeException(e);
            }
        }
        Map map = new HashMap();
        map.put(SystemConstant.TASK_ID, tbTaskHistory.getId());
        return ResultUtil.ok(map);
    }
}
